@@ -377,6 +377,8 @@
 #define DEFAULT_STATUS_NO_REPEAT 0
 #define DEFAULT_STATUS_OPER "*"
 #define DEFAULT_STATUS_OVERWRITE " (Overwrite)"
+#define DEFAULT_STATUS_PREFIX_WHEN_CURRENT ""
+#define DEFAULT_STATUS_PREFIX_WHEN_NOT_CURRENT ""
 #define DEFAULT_STATUS_QUERY " (Query: %Q)"
 #define DEFAULT_STATUS_SCROLLBACK " (Scroll)"
 #define DEFAULT_STATUS_SERVER " (%S)"
