@@ -33,25 +33,26 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+#define __need_term_h__
 #include "irc.h"
 
 
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-2.1.1";
-const char useful_info[] = "epic5 2 1 1";
+const char irc_version[] = "EPIC5-2.1.2";
+const char useful_info[] = "epic5 2 1 2";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20190224";
+const char internal_version[] = "20200511";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1898;
+const unsigned long	commit_id = 1908;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -59,7 +60,7 @@ const unsigned long	commit_id = 1898;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Abulia";
+const char ridiculous_version_name[] = "Lugubrious";
 
 #define __need_putchar_x__
 #include "status.h"
